import { useEffect, useState } from 'react';
import * as Y from 'yjs';
import { IndexeddbPersistence } from 'y-indexeddb'
import { WebsocketProvider } from 'y-websocket';

export function useYDoc(name: string) {
    const [ydoc] = useState<Y.Doc>(() => new Y.Doc());
    const [indexeddbProvider, setIndexeddbProvider] = useState<IndexeddbPersistence | null>(null);
    const [websocketProvider, setWebsocketProvider] = useState<WebsocketProvider | null>(null);

    useEffect(() => {
        if (!ydoc) {
            return
        }

        const indexeddbProvider = new IndexeddbPersistence(name, ydoc);
        setIndexeddbProvider(indexeddbProvider);

        const websocketProvider = new WebsocketProvider(`/sync`, name, ydoc, {
            connect: true,
        });
        setWebsocketProvider(websocketProvider);

        return () => {
            indexeddbProvider.destroy();
            websocketProvider.destroy();
            ydoc.destroy();
        };
    }, [ydoc]);

    return { ydoc, indexeddbProvider, websocketProvider };
}
